@@ -36,7 +36,7 @@ class Git(object):
     class Config(object):
         """Manage the git config for the current repository."""
         class ConfigSection(object):
-            """Managee a specific section of the git config."""
+            """Manage a specific section of the git config."""
             class ConfigItem(object):
                 """Represent a key:value (multi-)pair."""
                 def __init__(self, key):
@@ -146,8 +146,12 @@ class Git(object):
 
             def rm_items(self, key):
                 """Remove all values for the given key."""
-                del self._git._repo.config[self.itemname(key)]
                 del self._items[self.itemname(key)]
+                # No pygit2 interface for this.
+                prev_dir = Path.cwd()
+                os.chdir(self._git._repo.path)
+                capture_command(f'git config --unset-all {self.itemname(key)}')
+                os.chdir(prev_dir)
 
             def rm_item(self, key, pattern):
                 """Remove all values matching pattern from the given key."""
@@ -159,7 +163,10 @@ class Git(object):
                     del self._items[self.itemname(key)]
 
                 # No pygit2 interface for this.
+                prev_dir = Path.cwd()
+                os.chdir(self._git._repo.path)
                 capture_command(f'git config --unset {self.itemname(key)} {pattern}')
+                os.chdir(prev_dir)
 
             # No pygit2 interface for this.
             def rm(self):
@@ -357,12 +364,34 @@ class Git(object):
         descendant_oid = self._repo.revparse_single(descendant).id
         return self._repo.descendant_of(descendant_oid, committish_oid)
 
+    # Operations on remotes
+
+    def set_remote_fetch_refspecs(self, remote, refspecs):
+        """Set the refspec list for the given remote to refspecs."""
+        if isinstance(refspecs, str):
+            refspecs = [refspecs]
+
+        self.config.rm_items(f'remote.{remote}', 'fetch')
+        for refspec in refspecs:
+            self._repo.remotes.add_fetch(remote, refspec)
+
+    def get_remote_fetch_refspecs(self, remote):
+        """Get the list of refspecs for the given remote."""
+        return self._repo.remotes[remote].fetch_refspecs
+
+    def fetch_remote(self, remote, callbacks=None):
+        """Fetch all refspecs from the given remote."""
+        self._repo.remotes[remote].fetch(callbacks=callbacks)
+
     # Info on refs.
 
     @staticmethod
     def refname_to_branch_name(refname):
         """Translate a refname to a branch name."""
         prefix = 'refs/heads/'
+        if refname.startswith(prefix):
+            return refname[len(prefix):]
+        prefix = 'refs/remotes/'
         if refname.startswith(prefix):
             return refname[len(prefix):]
         return refname
@@ -451,13 +480,32 @@ class Git(object):
 
     def iterrefnames(self, patterns):
         """Iterate over all of the refnames matching the given pattern."""
+        if isinstance(patterns, str):
+            patterns = [patterns]
+
         for refname in self._repo.references:
             for pattern in patterns:
                 if refname.startswith(pattern):
                     yield refname
 
+    def iterbranches(self):
+        """Iterate over all of the repository's branches."""
+        for branch in self._repo.branches:
+            yield branch
+
     # Higher-level commands.
 
+    def detach_head(self):
+        """Cause HEAD to be detached."""
+        head_ref = self.get_committish_commit('HEAD')
+        if not self.is_bare_repository():
+            self._repo.checkout_tree(head_ref)
+        self._repo.set_head(head_ref.id)
+
+    def head_is_detached(self):
+        """Return whether HEAD is detched."""
+        return self._repo.head_is_detached
+
     def create_branch(self, branch_name, committish):
         """Create a branch with the given name pointing to committish."""
         commit = self.get_committish_commit(committish)
@@ -465,12 +513,35 @@ class Git(object):
         branch_name = self.refname_to_branch_name(branch_name)
         self._repo.branches.create(branch_name, commit)
 
-    def clone (self, url, bare=False, callbacks=None):
+    def set_branch_upstream(self, branch_name, remote_branch_name):
+        """Set the upstream of branch_name to remote_branch_name."""
+        branch_name = self.refname_to_branch_name(branch_name)
+        if remote_branch_name:
+            remote_branch_name = self.refname_to_branch_name(remote_branch_name)
+
+        branch = self._repo.branches[branch_name]
+
+        remote_branch = None
+        if remote_branch_name:
+            remote_branch = self._repo.branches[remote_branch_name]
+
+        branch.upstream = remote_branch
+
+    def get_branch_upstream(self, branch_name):
+        branch_name = self.refname_to_branch_name(branch_name)
+        branch = self._repo.branches[branch_name]
+        result = None
+        if branch.upstream:
+            result = branch.upstream.branch_name
+        return result
+
+    def clone(self, url, path=None, bare=False, callbacks=None):
         """Clone a respository at the given url, making a bare clone if specified."""
         parsed_url = urllib.parse.urlparse(url)
-        path = Path(parsed_url.path).resolve()
-        name = path.name
-        target_path = str(Path.cwd() / name)
+        url_path = Path(parsed_url.path).resolve()
+        url_name = url_path.name
+        target_path = path if path else str(Path.cwd() / url_name)
+
         self._repo = pygit2.clone_repository(url, target_path, bare, callbacks=callbacks)
         self._config = self.Config(self, self._repo.config)
 
@@ -521,6 +592,7 @@ class Git(object):
     def delete_branch(self, branch_name):
         """Delete the named branch."""
         # TODO: Check that it's actually a branch.
+        branch_name = self.refname_to_branch_name(branch_name)
         self._repo.branches.delete(branch_name)
 
     def remote_branch_exists(self, branch_name, remote):
